#include "windos.ch"
MemVar   PrtLPI,  PrtPitch,  PrtPaper,  PrtOrient, PrtReset
MemVar   nOrient, nPitch, nSpacing, cPrinter, cPaper
FUNCTION PrtSetup()
   LOCAL oPrtStack := WDStack():new()
      // Save current settings (LIFO)
      oPrtStack:push(nOrient)
      oPrtStack:push(nPitch)
      oPrtStack:push(nSpacing)
      oPrtStack:push(cPrinter)
      oPrtStack:push(cPaper)

      IF GetSetup()
        SaveSetup()
      ELSE
        cPaper   := oPrtStack:pop()
        cPrinter := oPrtStack:pop()
        nSpacing := oPrtStack:pop()
        nPitch   := oPrtStack:pop()
        nOrient  := oPrtStack:pop()
      ENDIF
   RETURN ( NIL )

FUNCTION GetSetup()
   LOCAL aOrient, aPitch, aSpacing, aPrinter, aPaper
   LOCAL oDlg
   LOCAL cBtnOk,  cBtnCancel
   LOCAL lOkClicked := .F.

      aOrient  := { {10,  3, "Portrait"   , "1" } , ;
                    {12,  3, "Landscape"  , "2" }  }

      aPitch   := { {10, 20, "10-Pitch"   , "1"} , ;
                    {11, 20, "12-Pitch"   , "2"} , ;
                    {12, 20, "Compressed" , "3"} }

      aSpacing := { {10, 38, "6 lpi"      , "1"} , ;
                    {12, 38, "8 lpi"      , "2"} }

      aPrinter := { {"Dot Matrix Generic" , "1" }, ;
                    {"HP-II Laser       " , "2" }, ;
                    {"HP-III Laser      " , "3" }, ;
                    {"LP-IIIsi Laser    " , "4" }, ;
                    {"LP-IV Laser       " , "5" }, ;
                    {"InkJet-500 Series " , "6" }, ;
                    {"InkJet-600 Series " , "7" }  }

      aPaper   := { {"8x11 Single Sheet" , "1" } ,;
                    {"8x14 Single Sheet" , "2" } ,;
                    {"8x11 Continuous  " , "3" } ,;
                    {"11x8 Continuous  " , "4" } ,;
                    {"14x8 Continuous  " , "5" } ,;
                    {"14x11 Continuous  " , "6" }  }



      DEFINE DIALOG oDlg FROM 0,0 TO 15,55 ;
         TITLE "Printer Setup" ;
         SYSMENU

      DEFINE COMBOBOX VAR cPrinter OF oDlg ;
         FROM 3,11 TO 10,37 ;
         ITEMS aPrinter ;
         MESSAGE "Choose Printer Type from the List" ;
         CAPTION "Printer:" ;
         SCROLLBAR

      DEFINE COMBOBOX VAR cPaper OF oDlg ;
         FROM 5,11 TO 12,37 ;
         ITEMS aPaper ;
         MESSAGE "Choose Paper Type from the List" ;
         CAPTION "Paper:" ;
         SCROLLBAR

      DEFINE RADIOGROUP VAR nOrient OF oDlg ;
         FROM 8,2 TO 13,17 ;
         ITEMS aOrient ;
         CAPTION "Orientation" ;
         MESSAGE "Choose Paper Orientation"

      DEFINE RADIOGROUP VAR nPitch OF oDlg ;
         FROM 8,19 TO 13,35 ;
         ITEMS aPitch ;
         CAPTION "Pitch" ;
         MESSAGE "Choose Printing Pitch"

      DEFINE RADIOGROUP VAR nSpacing OF oDlg ;
         FROM 8,37 TO 13,52 ;
         ITEMS aSpacing ;
         CAPTION "Spacing" ;
         MESSAGE "Choose Printer Spacing"

      DEFINE PUSHBUTTON VAR cBtnOk OF oDlg ;
         AT 3,42 ;
         CAPTION "Ok" ;
         ACTION {|| lOkClicked := .T., oDlg:end() } ;
         MESSAGE "Click Ok To Accept"

      DEFINE PUSHBUTTON VAR cBtnCancel OF oDlg ;
         AT 5,42 ;
         CAPTION "Cancel" ;
         ACTION {|| lOkClicked := .F., oDlg:end() } ;
         MESSAGE "Click Cancel To Quit"

      ACTIVATE DIALOG oDlg CENTERED

   RETURN ( lOkClicked )

PROCEDURE SaveSetup()
      DO CASE
        CASE cPrinter = "1"             // Dot Matrix
          PrtOrient := NIL
          PrtLPI     := NIL
          PrtPitch   := NIL
          PrtReset   := DM_RESET
        OTHERWISE                       // HP Compatible
          PrtReset := HP_RESET
          IF nOrient = 1
            PrtOrient := HP_PORTRAIT
          ELSE
            PrtOrient := HP_LANDSCAPE
          ENDIF
          IF nSpacing = 1
            PrtLPI := HP_6LPI
          ELSE
            PrtLPI := HP_8LPI
          ENDIF
          IF nPitch = 1
            PrtPitch := HP_10PITCH
          ELSEIF nPitch = 2
            PrtPitch := HP_12PITCH
          ELSE
            PrtPitch := HP_16PITCH
          ENDIF
          IF cPaper $ "13456"
            PrtPaper := HP_LETTER
          ELSE
            PrtPaper := HP_LEGAL
          ENDIF
      ENDCASE
   RETURN

FUNCTION ReportDlg(cTitle)
   LOCAL nResponse
     WHILE .T.
       nResponse := MsgOkCanSet(cTitle,"Print Report")
       IF nResponse = 3
         PrtSetup()
       ELSE
         EXIT
       ENDIF
     ENDDO
  RETURN (nResponse)
