#include "windos.ch"
#include "setcurs.ch"

/* This is a demo of WinDos53 with a mini billing application.

   The application illustrates most of the features of WinDos53.
   There are 8 program files associated with this demo and these are:

   WDDemo.prg              This program
   WDDCust.prg             Customer Maintenance
   WDDInv.prg              Inventory Maintenance
   WDDDisc.prg             Discount Code Table Maintenance
   WDDTerm.prg             Terms Code Table Maintenance
   WDDOrd.prg              Orders Data Entry
   WDDOrdPo.prg            Print Invoices from orders
   WDDInvVw.prg            Billing View

   Also associated with this demo are the following tables and indexes

   WDCUST   DBF                    WDCUST   NTX
   WDFIN    DBF                    WDFIN    NTX
   WDNOTE   DBF    WDNOTE   DBT    WDNOTE   NTX

   WDTERMS  DBF                    WDTERMS  NTX

   WDDISC   DBF                    WDDISC   NTX

   WDORDH   DBF                    WDORDH   NTX

   WDORDD   DBF                    WDORDD1 NTX
                                   WDORDD2 NTX

   WDINV    DBF    WDINV    DBT    WDINV    NTX

   WDACCTS  DBF                    WDACCTS1 NTX
                                   WDACCTS2 NTX

*/

***********************************************************************
* This is the main root program... That's all there is to it!
*
* All it does is set up the envirnoment (the SET commands) ;
*   loads our font we want to use; define the window and its
*   menu; and activates the window.
***********************************************************************
// PUBLIC Declarations used by all print function
//    these are the only public variables that are used (ugh!) but
//    unfortunately, are necessary.
MemVar   PrtLPI,  PrtPitch,  PrtPaper,  PrtOrient, PrtReset
MemVar   nOrient, nPitch, nSpacing, cPrinter, cPaper
***********************************************************************
PROCEDURE Main
   LOCAL oWnd, aFont, aWDLogo
   PUBLIC PrtLPI,  PrtPitch,  PrtPaper,  PrtOrient, PrtReset
   PUBLIC nOrient, nPitch, nSpacing, cPrinter, cPaper

      SetCursor( SC_NONE )

      // You can try this application without the next statement
      //   it will just go into text mode
      SET VIDEOMODE TO LLG_VIDEO_VGA_640_480_16

      // Establish mouse events
      SET( _SET_EVENTMASK, INKEY_ALL )
      MSetCursor(.T.)
      SET SCOREBOARD ON
      SET DELETED ON
      Set ( _SET_ESCAPE , .F. )

      // Set the file path
      SET PATH TO \WINDOS53\DATA

      // We have an ini file attached to this demo that contains
      //    the printer defaults.
      SET PROPERTIES TO WDDEMO.INI

      SetPrtDefault()

      SET 3DLOOK ON

      // Load the font into array
      LOAD FONT "NORMS.FND"  TO aFont

      // Define the Window
      DEFINE WINDOW oWnd FROM 0,0 TO MaxRow(),MaxCol() ;
         TITLE "WinDos53 Demo" ;
         FRAME TYPE NONE ;
         MENU SetMenu(oWnd) ;
         SYSMENU

      // Ok....Let's get under way
      ACTIVATE WINDOW oWnd

      // Destroy the font
      RELEASE FONT aFont
      SetCursor( SC_NORMAL )
   RETURN

***********************************************************************
PROCEDURE SetPrtDefault()
   LOCAL oIni := WDProperty():new( _Properties() )
      // Read the ini file and get the printer defaults
      oIni:load()
      cPrinter  := oIni:getAttrib("PRINTER","printer")
      cPaper    := oIni:getAttrib("PRINTER","paper")
      nPitch    := Val( oIni:getAttrib("PRINTER","pitch") )
      nOrient   := Val( oIni:getAttrib("PRINTER","orient") )
      nSpacing  := Val( oIni:getAttrib("PRINTER","spacing") )
      cPrinter  := IIf(cPrinter==NIL,"1",cPrinter)
      cPaper    := IIf(cPaper==NIL,"1", cPaper)
      nPitch    := IIf(nPitch==NIL,1,nPitch)
      nOrient   := IIf(nOrient==NIL,1,nOrient)
      nSpacing  := IIf(nSpacing==NIL,1,nSpacing)
      SaveSetUp()
   RETURN
***********************************************************************
STATIC FUNCTION SetMenu(oWnd)
   LOCAL oMenu, oPopUp, oPopUp1

      // Define our menu for the shell window

      // Start with your menu bar definition
      DEFINE MENUBAR oMenu COLOR "N/W,W/B,R/W,R+/B,N+/W,N/W" ;
        SELECT 0

      // Define the File popup menu in the menubar
      DEFINE POPUPMENU oPopUp COLOR "N/W,W/B,R/W,R+/B,N+/W,N/W"
        DEFINE POPUPITEM OF oPopUp ;
          CAPTION "Print &Setup.." ;
          ACTION  {|| PrtSetup() } ;
          MESSAGE "Set Up Your Printer" ;
          ID 101

        DEFINE MENU SEPARATOR OF oPopUp

        DEFINE POPUPITEM OF oPopUp ;
          CAPTION "E&xit" ;
          ACTION oWnd:end() ;
          MESSAGE "Exit this program" ;
          ID 102

        DEFINE MENUITEM OF oMenu ;
          CAPTION "&File" ;
          POPUP oPopUp ;
          MESSAGE "File Operations" ;
          ID 100

      // Define the Tables menu in the menubar
      DEFINE POPUPMENU oPopUp COLOR "N/W,W/B,R/W,R+/B,N+/W,N/W"
        DEFINE POPUPMENU oPopUp1 COLOR "N/W,W/B,R/W,R+/B,N+/W,N/W"

        DEFINE POPUPITEM OF oPopUp ;
          CAPTION "&Customers" ;
          ACTION {|| WDDCust() } ;
          MESSAGE "Capture Customer Data" ;
          ID 201

        DEFINE POPUPITEM OF oPopUp ;
          CAPTION "&Inventory" ;
          ACTION {|| WDDInv() } ;
          MESSAGE "Capture Inventory Data" ;
          ID 202

        DEFINE POPUPITEM OF oPopUp ;
          CAPTION "&Discounts" ;
          ACTION {|| WDDDisc() } ;
          MESSAGE "Capture Discount Code Table Data" ;
          ID 203

        DEFINE POPUPITEM OF oPopUp ;
          CAPTION "&Terms" ;
          ACTION {|| WDDTerm() } ;
          MESSAGE "Capture Terms Code Table Data" ;
          ID 204

        DEFINE MENUITEM OF oMenu ;
          CAPTION "&Tables" ;
          POPUP oPopUp ;
          MESSAGE "Table Maintenance" ;
          ID 200

      // Define the Orders menu in the menubar
      DEFINE POPUPMENU oPopUp COLOR "N/W,W/B,R/W,R+/B,N+/W,N/W"
        DEFINE POPUPMENU oPopUp1 COLOR "N/W,W/B,R/W,R+/B,N+/W,N/W"

        DEFINE POPUPITEM OF oPopUp ;
          CAPTION "Data Entry" ;
          ACTION {|| WDDOrd() } ;
          MESSAGE "Capture Customer Orders for Billing" ;
          ID 301

        DEFINE POPUPITEM OF oPopUp ;
          CAPTION "Print" ;
          ACTION {|| WDDOrdPo() } ;
          MESSAGE "Produce Invoices" ;
          ID 302

        DEFINE MENUITEM OF oMenu ;
          CAPTION "&Orders" ;
          POPUP oPopUp ;
          MESSAGE "Order Entry Operations" ;
          ID 300

      // Define the Invoices menu in the menubar
      DEFINE POPUPMENU oPopUp COLOR "N/W,W/B,R/W,R+/B,N+/W,N/W"
        DEFINE POPUPMENU oPopUp1 COLOR "N/W,W/B,R/W,R+/B,N+/W,N/W"

        DEFINE POPUPITEM OF oPopUp ;
          CAPTION "View" ;
          ACTION {|| WDDInvVw() } ;
          MESSAGE "View Customer Billing Data" ;
          ID 401

        DEFINE MENUITEM OF oMenu ;
          CAPTION "&Invoices" ;
          POPUP oPopUp ;
          MESSAGE "Customer Invoice Procedures" ;
          ID 400

   RETURN ( oMenu )
***********************************************************************
* These procedures are common procedures used from various programs
*  in the application
***********************************************************************
// Procedures to Save Modify, Save Add, Delete Records
FUNCTION DelRecord(oDbf)
      MsgTone()
      IF MsgOkCan("Delete Item","Delete this Item") = BOX_REPLY_OKAY
        oDbf:delete()
        MsgFlash("Delete Record","Record Deleted")
      ENDIF
   RETURN ( NIL )

FUNCTION SaveRecord( lNew, oDlg, oDbf )
      IF lNew
        oDbf:append()
        oDbf:save()
        oDlg:end()
      ELSE
        oDbf:save()
      ENDIF
   RETURN ( .T. )

FUNCTION GetNextCode( oDbf )
   LOCAL nRec := oDbf:recno(), cNextCode
      oDbf:gobottom()
      cNextCode := StrZero( Val( (oDbf:alias)->code) + 1, 2 )
      oDbf:goto(nRec)
   RETURN ( cNextCode )

