#include "windos.ch"
#include "getexit.ch"

MemVar   PrtLPI,  PrtPitch,  PrtPaper,  PrtOrient, PrtReset

STATIC oBrw
STATIC oCustData, oCustFin, oOrdHdr, oOrdDetl
STATIC oTerms, oDisc, oInvent, oArData
STATIC nTotalOrd, nTotalDisc, nNetOrder

/***

  This program produces the customer invoices from the pending order
  file.  This program uses the report writer to print both the
  invoices and the invoice register report.

  The program is quite busy in terms of tables and relations.  Note the
  relation tree moves from the bottom up and always is a many-to-one
  relationship in all cases.
***/

PROCEDURE WDDOrdPo
   LOCAL nResponse, nFirstInv

      // Open the tables required for this operation
      //    and set the relations
      DEFINE TABLE WdTerms AS oTerms    INDEX WdTerms NEW
      DEFINE TABLE WdDisc  AS oDisc     INDEX WdDisc NEW
      DEFINE TABLE WdFin   AS oCustFin  INDEX WdFin NEW ;
         RELATE TO WdTerms ON WdFin->Terms ;
                TO WdDisc ON WdFin->Disc
      DEFINE TABLE WdCust  AS oCustData INDEX WdCust NEW ;
         RELATE TO WdFin ON WdCust->CustNo
      DEFINE TABLE WdInv   AS oInvent   INDEX WdInv   NEW
      DEFINE TABLE WdOrdH AS oOrdHdr    INDEX WdOrdH NEW ;
         RELATE TO WdCust ON WdOrdH->CustNo
      DEFINE TABLE WdOrdD AS oOrdDetl   INDEX WdOrdD1, WdOrdD2 NEW ;
         RELATE TO WdOrdH ON WdOrdD->CustNo+WdOrdD->Orderno ;
         RELATE TO WdInv   ON WdOrdD->Partno
      DEFINE TABLE WdAccts AS oArData   INDEX WdAccts1, WdAccts2 NEW ;
         RELATE TO WdCust ON WdAccts->CustNo

      // Check if any orders to invoice
      IF oOrdHdr:eof()
        MsgBeep()
        MsgStop("Print Invoices","No Orders to Invoice")
        CLOSE DATABASES
        RETURN
      ENDIF

      // Check if user wishes to proceed with print operation
      //  and if so, print the order, then print the register.
      nResponse := ReportDlg("Print Invoices")
      IF nResponse = 1
        nFirstInv := WDDOrdSumm()
        WDDInvPrnt()
        WDDRegPrnt(nFirstInv)
      ENDIF

      CLOSE DATABASES

      // Delete all order records
      //   We have to open exclusive to do this operation.
      DEFINE TABLE WdOrdD   AS oOrdDetl INDEX WdOrdD1, WdOrdD2 NEW EXCLUSIVE
      DEFINE TABLE WdOrdH   AS oOrdHdr  INDEX WdOrdH    NEW EXCLUSIVE
      oOrdDetl:zap()
      oOrdHdr:zap()
      CLOSE DATABASES
   RETURN

FUNCTION WddOrdSumm()
   LOCAL nLineAmt, nGross, nNet, nInvoice, nDisc
   LOCAL nFirstInv := 0
      // Set A/R data to invoice number order
      oArData:setorder(2)
      // Create the Accounts Receivable Summary Records
      oOrdDetl:select()
      oOrdDetl:gotop()
      WHILE !( oOrdDetl:eof() )
        oOrdDetl:load()
        oCustFin:load()
        oOrdHdr:load()
        nGross := nNet := 0

        // Get the next A/R Invoice Number
        IF !( oArData:eof() )
          oArData:goBottom()
        ENDIF
        oArData:load()
        oArData:InvoiceNo ++
        IF nFirstInv = 0
          nFirstInv := oArData:InvoiceNo
        ENDIF
        // Update the Order Header with the invoice number
        oOrdHdr:InvoiceNo := oArData:InvoiceNo
        oOrdHdr:save()
        WHILE oOrdDetl:custno+oOrdDetl:orderno = WdOrdD->Custno+WdOrdD->OrderNo
          oOrdDetl:load()
          nLineAmt := oOrdDetl:qty * oInvent:unitprice
          nGross += nLineAmt
          oOrdDetl:skip()
        ENDDO
        nDisc := Round((nGross * oDisc:percent) / 100,2)
        nNet := nGross - nDisc
        oArData:custno := oOrdDetl:custno
        oArData:date   := Date()
        oArData:gross  := nGross
        oArData:disc   := nDisc
        oArData:net    := nNet
        oArData:append()
        oArData:save()
      ENDDO
      // Return A/R Data to normal index order
      oArData:setorder(1)
    RETURN ( nFirstInv )


PROCEDURE WDDInvPrnt()
   LOCAL oInvoice
/******
*   This module gives an example on the flexibility of the
*   WinDos53 report writer.  It is not just a tool for printing
*   reports but can be used for many other purposes such as this
******/

      oOrdDetl:gotop()

      // Define the report object
      DEFINE REPORT oInvoice ;
         MAXLINES 50 ;
         ORIENT PrtOrient ;
         PAPER SIZE PrtPaper ;
         PrtLPI LPI ;
         RESET CODE PrtReset

      // Define the report group
      DEFINE REPORT GROUP OF oInvoice ;
         LEVEL 1 ;
         FIELD WdOrdD->CustNo+WdOrdD->OrderNo ;
         BLANK LINE BEFORE FOOTING

      // Internal calculations

      DEFINE TOTAL FIELD InvGross OF oInvoice ;
         SUM WdOrdD->total ;
         RESET 1

      DEFINE CALC FIELD InvDisc OF oInvoice ;
         EXPRESSION Round((InvGross*WdDisc->Percent)/100,2)

      DEFINE CALC FIELD InvNet OF oInvoice ;
         EXPRESSION InvGross-InvDisc

      // The Invoice
      // ***********

      // Page Headings
      DEFINE PAGE HEADING OF oInvoice ;
         LINE 1 ;
         COLUMN 0 ;
         VALUE Padc("Universal Supply, Inc.",80)

      DEFINE PAGE HEADING OF oInvoice ;
         LINE 2 ;
         COLUMN 0 ;
         VALUE Padc("432 Maple Street",80)

      DEFINE PAGE HEADING OF oInvoice ;
         LINE 3 ;
         COLUMN 0 ;
         VALUE Padc("Smallville, OH 12345-1234",80)

      DEFINE PAGE HEADING OF oInvoice ;
         LINE 5 ;
         COLUMN 0 ;
         VALUE Padc("I N V O I C E",80)

      // Group Headings
      // Notice that we start the group headings at line number 2
      //  so that we have a blank line following the last page heading
      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 2 ;
         COLUMN 0 ;
         VALUE "Sold To:"

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 2 ;
         COLUMN 9 ;
         FIELD WdCust->Name

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 2 ;
         COLUMN 50 ;
         VALUE "Date:"

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 2 ;
         COLUMN 58 ;
         FIELD Date()

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 3 ;
         COLUMN 9 ;
         FIELD WdCust->Address

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 3 ;
         COLUMN 50 ;
         VALUE "Number:"

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 3 ;
         COLUMN 58 ;
         FIELD WdOrdH->InvoiceNo ;
         PICTURE "99999"

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 4 ;
         COLUMN 9 ;
         FIELD WdCust->City

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 4 ;
         COLUMN 50 ;
         VALUE "Terms:"

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 4 ;
         COLUMN 58 ;
         FIELD WdTerms->Desc

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 5 ;
         COLUMN 9 ;
         FIELD WdCust->Prov

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 5 ;
         COLUMN 15 ;
         FIELD WdCust->Postcode

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 7 ;
         COLUMN 0 ;
         VALUE "ITEM   QTY   DESCRIPTION                 PRICE     TOTAL"

      DEFINE GROUP HEADING OF oInvoice ;
         LEVEL 1 ;
         LINE 8 ;
         COLUMN 0 ;
         VALUE "====   ===   ========================= =======  ========"

      // The Invoice Details
      DEFINE BODY OF oInvoice ;
         LINE 1 ;
         COLUMN 0 ;
         FIELD WdOrdD->PartNo

      DEFINE BODY OF oInvoice ;
         LINE 1 ;
         COLUMN 7 ;
         FIELD WdOrdD->Qty ;
         PICTURE "999"

      DEFINE BODY OF oInvoice ;
         LINE 1 ;
         COLUMN 13 ;
         FIELD WdInv->Desc

      DEFINE BODY OF oInvoice ;
         LINE 1 ;
         COLUMN 39 ;
         FIELD WdInv->UnitPrice ;
         PICTURE "9999.99"

      DEFINE BODY OF oInvoice ;
         LINE 1 ;
         COLUMN 48 ;
         FIELD WdOrdD->total ;
         PICTURE "99999.99"

      // Invoice Footings
      DEFINE GROUP FOOTING OF oInvoice ;
         LEVEL 1 ;
         LINE 1 ;
         COLUMN 0 ;
         VALUE "Less "

      DEFINE GROUP FOOTING OF oInvoice ;
         LEVEL 1 ;
         LINE 1 ;
         COLUMN 6 ;
         FIELD WdDisc->Desc

      DEFINE GROUP FOOTING OF oInvoice ;
         LEVEL 1 ;
         LINE 1 ;
         COLUMN 48 ;
         FIELD InvDisc ;
         PICTURE "@R 99999.99CR"

      DEFINE GROUP FOOTING OF oInvoice ;
         LEVEL 1 ;
         LINE 3 ;
         COLUMN 0 ;
         VALUE "Total Due"

      DEFINE GROUP FOOTING OF oInvoice ;
         LEVEL 1 ;
         LINE 3 ;
         COLUMN 48 ;
         FIELD InvNet ;
         PICTURE "99999.99"

      DEFINE PAGE BREAK OF oInvoice ;
         LEVEL 1

      ACTIVATE REPORT oInvoice

PROCEDURE WDDRegPrnt(nFirstInv)
   LOCAL oRegister

      // Another report writer example - this time for a simple
      //   report with final totals.
      oArData:select()
      SET FILTER TO WdAccts->InvoiceNo >= nFirstInv

      // Define the report object
      DEFINE REPORT oRegister ;
         MAXLINES 50 ;
         ORIENT PrtOrient ;
         PAPER SIZE PrtPaper ;
         PrtLPI LPI ;
         RESET CODE PrtReset

      DEFINE TOTAL FIELD TotalGross OF oRegister ;
         SUM WdAccts->Gross ;
         RESET 6

      DEFINE TOTAL FIELD TotalDisc OF oRegister ;
         SUM WdAccts->Disc ;
         RESET 6

      DEFINE TOTAL FIELD TotalSales OF oRegister ;
         SUM WdAccts->Net ;
         RESET 6

      // Page Headings
      DEFINE PAGE HEADING OF oRegister ;
         LINE 1 ;
         COLUMN 0 ;
         VALUE Padc("Universal Supply, Inc.",80)

      DEFINE PAGE HEADING OF oRegister ;
         LINE 2 ;
         COLUMN 0 ;
         VALUE Padc("Invoice Register",80)

      DEFINE PAGE HEADING OF oRegister ;
         LINE 4 ;
         COLUMN 0 ;
         VALUE "CUST  NAME                      INVOICE   DATE     GROSS  DISCOUNT   NET   "

      DEFINE PAGE HEADING OF oRegister ;
         LINE 5 ;
         COLUMN 0 ;
         VALUE "====  ========================= ======= ======== ======== ======== ========"

      DEFINE BODY OF oRegister ;
         LINE 1 ;
         COLUMN 0 ;
         FIELD WdAccts->CustNo

      DEFINE BODY OF oRegister ;
         LINE 1 ;
         COLUMN 6 ;
         FIELD WdCust->Name

      DEFINE BODY OF oRegister ;
         LINE 1 ;
         COLUMN 32 ;
         FIELD WdAccts->InvoiceNo ;
         PICTURE "99999"

      DEFINE BODY OF oRegister ;
         LINE 1 ;
         COLUMN 40 ;
         FIELD WdAccts->Date

      DEFINE BODY OF oRegister ;
         LINE 1 ;
         COLUMN 49 ;
         FIELD WdAccts->Gross ;
         PICTURE "99999.99"

      DEFINE BODY OF oRegister ;
         LINE 1 ;
         COLUMN 58 ;
         FIELD WdAccts->Disc ;
         PICTURE "99999.99"

      DEFINE BODY OF oRegister ;
         LINE 1 ;
         COLUMN 67 ;
         FIELD WdAccts->Net ;
         PICTURE "99999.99"

      DEFINE GROUP FOOTING OF oRegister ;
         LEVEL 6 ;
         LINE 2 ;
         COLUMN 30 ;
         VALUE "Totals:"

      DEFINE GROUP FOOTING OF oRegister ;
         LEVEL 6 ;
         LINE 2 ;
         COLUMN 49;
         FIELD TotalGross ;
         PICTURE "99999.99"

      DEFINE GROUP FOOTING OF oRegister ;
         LEVEL 6 ;
         LINE 2 ;
         COLUMN 58;
         FIELD TotalDisc ;
         PICTURE "99999.99"

      DEFINE GROUP FOOTING OF oRegister ;
         LEVEL 6 ;
         LINE 2 ;
         COLUMN 67;
         FIELD TotalSales ;
         PICTURE "99999.99"

      ACTIVATE REPORT oRegister

   RETURN
